MODULE dtasshd_surf
   !!======================================================================
   !!                     ***  MODULE  dtasshd_surf  ***
   !! Ocean data  :  read ocean SSH current data from gridded data
   !!======================================================================
   !! History :  3.6   ! 
   !!----------------------------------------------------------------------

   !!----------------------------------------------------------------------
   !!   dta_sshd_init_surf  : read namelist and allocate data structures
   !!   dta_sshd_surf       : read and time-interpolate ocean U & V current data
   !!----------------------------------------------------------------------
   USE oce             ! ocean dynamics and tracers
   USE dom_oce         ! ocean space and time domain
   USE fldread         ! read input fields
   USE in_out_manager  ! I/O manager
   USE phycst          ! physical constants
   USE lib_mpp         ! MPP library
   USE wrk_nemo        ! Memory allocation
   USE timing          ! Timing

   IMPLICIT NONE
   PRIVATE

   PUBLIC   dta_sshd_init_surf   ! called by istate.F90
   PUBLIC   dta_sshd_surf        ! called by istate.F90 and dyndmp.90

   LOGICAL , PUBLIC ::   ln_sshd_init      ! Flag to initialise with SSH data
   LOGICAL , PUBLIC ::   ln_sshd_sshdmp    ! Flag for Newtonian damping toward SSH data

   TYPE(FLD), ALLOCATABLE, DIMENSION(:) ::   sf_sshd   ! structure for input SSH (file information and data)

   !! * Substitutions
#  include "domzgr_substitute.h90"
   !!----------------------------------------------------------------------
   !! NEMO/OPA 3.3 , NEMO Consortium (2010)
   !! $Id: dtasshd_surf.F90 
   !! Software governed by the CeCILL licence     (NEMOGCM/NEMO_CeCILL.txt)
   !!----------------------------------------------------------------------
CONTAINS

   SUBROUTINE dta_sshd_init_surf(ld_sshdmp)
      !!----------------------------------------------------------------------
      !!                   ***  ROUTINE dta_sshd_init_surf  ***
      !!                    
      !! ** Purpose :   initialization of SSH input data 
      !! 
      !! ** Method  : - read namsshd namelist
      !!              - allocate SSH data structure
      !!              - fld_fill data structure with namelist information
      !!----------------------------------------------------------------------
      LOGICAL, INTENT(in), OPTIONAL ::   ld_sshdmp         ! force the initialization when sshdmp is used
      !
      INTEGER ::   ierr0, ierr1                            ! temporary integers
      !
      CHARACTER(len=100)            ::   cn_dir            ! Root directory for location of files to be used
      TYPE(FLD_N), DIMENSION(1)     ::   sssh_i            ! array of namelist informations on the fields to read
      TYPE(FLD_N)                   ::   sn_ssh            ! SSH data namelist information
      !!
      NAMELIST/namsshd/ ln_sshd_init, ln_sshd_sshdmp, cn_dir, sn_ssh
      INTEGER  ::   ios
      !!----------------------------------------------------------------------
      !
      IF( nn_timing == 1 )  CALL timing_start('dta_sshd_init_surf')
      !
      !  Initialisation
      ierr0 = 0  ;  ierr1 = 0

      REWIND( numnam_ref )              ! Namelist namsshd in reference namelist : 
      READ  ( numnam_ref, namsshd, IOSTAT = ios, ERR = 901)
901   IF( ios /= 0 ) CALL ctl_nam ( ios , 'namsshd in reference namelist', lwp )

      REWIND( numnam_cfg )              ! Namelist namsshd in configuration namelist : Parameters of the run
      READ  ( numnam_cfg, namsshd, IOSTAT = ios, ERR = 902 )
902   IF( ios /= 0 ) CALL ctl_nam ( ios , 'namsshd in configuration namelist', lwp )
      IF(lwm) WRITE ( numond, namsshd )

      !                             ! force the initialization when sshdmp is used
      IF( PRESENT( ld_sshdmp ) )   ln_sshd_sshdmp = .TRUE.
      
      IF(lwp) THEN                  ! control print
         WRITE(numout,*)
         WRITE(numout,*) 'dta_sshd_init_surf : SSH data '
         WRITE(numout,*) '~~~~~~~~~~~~ '
         WRITE(numout,*) '   Namelist namsshd : Set flags'
         WRITE(numout,*) '      Initialization of ocean SSH with input data   ln_sshd_init   = ', ln_sshd_init
         WRITE(numout,*) '      Damping of ocean SSH toward input data        ln_sshd_sshdmp = ', ln_sshd_sshdmp
         WRITE(numout,*)
         IF( .NOT. ln_sshd_init .AND. .NOT. ln_sshd_sshdmp ) THEN
            WRITE(numout,*)
            WRITE(numout,*) '   SSH data not used'
         ENDIF
      ENDIF
      !                             ! no initialization when restarting
      IF( ln_rstart .AND. ln_sshd_init ) THEN
         CALL ctl_warn( 'dta_sshd_init_surf: ocean restart and SSH data initialization, ',   &
            &           'we keep the restart SSH values and set ln_sshd_init to FALSE' )
         ln_sshd_init = .FALSE.
      ENDIF
      !
      !                             ! allocate the arrays (if necessary)
      IF(  ln_sshd_init .OR. ln_sshd_sshdmp  ) THEN

         ALLOCATE( sf_sshd(1), STAT=ierr0 )
         IF( ierr0 > 0 ) THEN
            CALL ctl_stop( 'dta_sshd_init_surf: unable to allocate sf_sshd structure' )             ;   RETURN
         ENDIF
         !
                                ALLOCATE( sf_sshd(1)%fnow(jpi,jpj,1)   , STAT=ierr0 )
         IF( sn_ssh%ln_tint )   ALLOCATE( sf_sshd(1)%fdta(jpi,jpj,1,2) , STAT=ierr1 )
         !
         IF( ierr0 + ierr1 > 0 ) THEN
            CALL ctl_stop( 'dta_sshd_init_surf : unable to allocate SSH data arrays' )   ;   RETURN
         ENDIF
         !                          !==   fill sf_sshd with sn_ssh and control print   ==!
         sssh_i(1) = sn_ssh
         CALL fld_fill( sf_sshd, sssh_i, cn_dir, 'dta_sshd_surf', 'SSH data', 'namsshd' )
         !
      ENDIF
      !
      IF( nn_timing == 1 )  CALL timing_stop('dta_sshd_init_surf')
      !
   END SUBROUTINE dta_sshd_init_surf


   SUBROUTINE dta_sshd_surf(kt,psshd)
      !!----------------------------------------------------------------------
      !!                   ***  ROUTINE dta_sshd_surf  ***
      !!                    
      !! ** Purpose :   provides SSH data at time step kt
      !! 
      !! ** Method  : - call fldread routine
      !!              - ORCA_R2: make some hand made alterations to the data (EMPTY)
      !!              - s- or mixed s-zps coordinate: vertical interpolation onto model mesh
      !!              - zps coordinate: vertical interpolation onto last partial level
      !!              - ln_sshd_sshdmp=False: deallocate the SSH data structure,
      !!                                      as the data is no longer used
      !!
      !! ** Action  :   psshd,  SSH data interpolated onto model mesh at time-step kt
      !!----------------------------------------------------------------------
      INTEGER                       , INTENT(in   ) ::   kt     ! ocean time-step
      REAL(wp), DIMENSION(jpi,jpj),   INTENT(  out) ::   psshd  ! SSH data
      !!----------------------------------------------------------------------
      !
      IF( nn_timing == 1 )  CALL timing_start('dta_sshd_surf')
      !
      CALL fld_read( kt, 1, sf_sshd )      !==   read SSH data at time step kt   ==!
      !
      psshd(:,:) = sf_sshd(1)%fnow(:,:,1) * tmask(:,:,1)       ! apply mask
      !
      IF( lwp .AND. kt == nit000 ) THEN   ! control print
         WRITE(numout,*) ' SSH '
         WRITE(numout,*)
         CALL prihre( psshd(:,:), jpi, jpj, 1, jpi, 20, 1, jpj, 20, 1., numout )
         WRITE(numout,*)
      ENDIF
      !
      IF( .NOT. ln_sshd_sshdmp    ) THEN   !==   deallocate SSH structure   ==! 
         !                                 !==   (data used only for initialization)  ==!
         IF(lwp) WRITE(numout,*) 'dta_sshd_surf: deallocate SSH arrays as they are only used to initialize the run'
                                   DEALLOCATE( sf_sshd(1)%fnow )     ! SSH arrays in the structure
         IF( sf_sshd(1)%ln_tint )  DEALLOCATE( sf_sshd(1)%fdta )
                                  
                                   DEALLOCATE( sf_sshd         )     ! the structure itself
      ENDIF
      !
      IF( nn_timing == 1 )  CALL timing_stop('dta_sshd_surf')
      !
   END SUBROUTINE dta_sshd_surf

   !!======================================================================
END MODULE dtasshd_surf
